import datetime
import os

class File:
    """Filename representation, with possible size and
    modification time information.
    
    :ivar abs_filename: Absolute filename as string. The filename
      is normalized such that it always contains forward slashes,
      even on Windows.
    """

    def __init__(self, filename):
        self.abs_filename = filename

        if os.path.abspath(filename).replace('\\', '/') != filename:
            raise ValueError, 'Filename must be absolute and normalized ("%s")' % filename
        if not os.path.exists(filename):
            raise ValueError, 'File "%s" not found' % filename

        self._size = None
        self._modtime = None
        self._isfile = os.path.isfile(self.abs_filename)
        """
        if os.path.isfile(self.abs_filename):
            self.size = os.path.getsize(self.abs_filename)
            timestamp = os.path.getmtime(self.abs_filename)
            try:
                self.modtime = datetime.datetime.fromtimestamp(timestamp)
            except ValueError:
                # datetime raises error for out-of-bounds dates
                self.modtime = None
        else:
            self.size = None
            self.modtime = None
        """

    def __cmp__(self, other):
        if not isinstance(other, File):
            return -1
        return cmp(self.abs_filename, other.abs_filename)

    def __str__(self):
        return self.abs_filename

    def __hash__(self):
        return hash(self.abs_filename)

    def get_size(self):
        """Return size of file in bytes. Return None if the file
        is a directory."""
        if self._size is None and self._isfile:
            self._size = os.path.getsize(self.abs_filename)
        return self._size

    def get_modtime(self):
        """Return modification time as `datetime.datetime` instance.
        Return None if the file is a directory or the modification
        time can't otherwise be determined."""

        if self._modtime is None and self._isfile:
            timestamp = os.path.getmtime(self.abs_filename)
            try:
                self._modtime = datetime.datetime.fromtimestamp(timestamp)
            except ValueError:
                # datetime raises error for out-of-bounds dates
                self._modtime = None
        return self._modtime

    def calc_relative_filename(self, rootdir):
        """Return a relative filename r so that rootdir+r is
        the absolute filename. May return None if the
        relative filename can't be calculated."""

        rootdir_lower = os.path.abspath(rootdir).replace('\\', '/').lower()
        absname_lower = self.abs_filename.lower()
        if not absname_lower.startswith(rootdir_lower):
            return None

        relname = self.abs_filename[len(rootdir_lower):]
        if relname.startswith('/'):
            relname = relname[1:]
        if not relname:
            relname = '.'
        return relname
