class Visitor:
    """
    Abstract base class for visitor objects. The visitor
    mechanism is implemented by a combination of the AST.ASTNode.visit
    method and this class.
    
    This base class defines skeleton methods that must be
    overridden in concrete visitor classes. Each visit_XXX method
    receives an astnode (AST.ASTNode instance) object as argument
    and an additional arg argument that may be used to pass further
    relevant information.
    """

    def visit_BlockRule(self, astnode, arg):
        raise NotImplementedError

    def visit_GloblistRule(self, astnode, arg):
        raise NotImplementedError

    def visit_ForEachRule(self, astnode, arg):
        raise NotImplementedError

    def visit_GlobPattern(self, astnode, arg):
        raise NotImplementedError

    def visit_AndExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_OrExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_NotExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_FunctionExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_CompareExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_LiteralExpression(self, astnode, arg):
        raise NotImplementedError

    def visit_VariableExpression(self, astnode, arg):
        raise NotImplementedError
