"""
Classes for runtime variables and parse-time symbol table.
"""

import AST

class Variable:
    """
    Runtime variable with name, type and value.
    The name is stored in 'variable_name', type in
    'variable_type' (one of globals.E_*) and
    value is 'value'.
    """
    
    def __init__(self, variable_name, variable_type):
        self.variable_name = variable_name
        self.variable_type = variable_type
        self.value = None

    def set_value(self, value):
        self.value = value

    def __str__(self):
        return 'Variable (%s, type %s)' % (self.variable_name, AST.type2str(self.variable_type))


class SymbolTable:
    """
    Stack-like 'symbol table' that is used during parsing. The
    table is not used after the parsing phase.
    """
    
    def __init__(self):
        self._stack = [] # [Variable]

    def push(self, variable):
        """Add a Variable instance to the top of the stack."""
        # This gives O(n) instead of O(1)...
        self._stack.insert(0, variable)

    def pop(self):
        """Return the top-most Variable instance."""
        self._stack.pop(0)

    def get(self, symbolname):
        """Return the current (active) Variable instance for the
        name 'symbolname'. If no such variable exists, return None."""
        
        for variable in self._stack:
            if variable.variable_name == symbolname:
                return variable
        return None